import 'dart:io';

import 'package:flutter/material.dart';
import 'package:path_provider/path_provider.dart';
import 'package:toast/toast.dart';
import 'package:vais_speech/vais.dart';
import 'package:vais_speech_example/recording_state.dart';

const String name = "add_your_path_here.aac";

class RecordScreen extends StatefulWidget {
  final VaisSpeechAudio vaisSpeechAudio;
  final VaisSpeech vaisSpeech;
  final bool isOfflineMode;

  const RecordScreen({Key key, @required this.vaisSpeechAudio, this.isOfflineMode = true, this.vaisSpeech}) : super(key: key);

  @override
  _RecordScreenState createState() => _RecordScreenState();
}

class _RecordScreenState extends State<RecordScreen> {
  String _text;
  Duration _duration;
  RecordingState recordingState;
  String _path;

  @override
  void initState() {
    _init();
    super.initState();
  }
  
  Future _init() async {
     _path = await getPath();
    await widget.vaisSpeechAudio.init(_path, recordMode: widget.isOfflineMode ? RecordMode.Offline : RecordMode.Streaming);
    print("Recorder initilized with ${widget.vaisSpeechAudio.recordMode} mode !");
    widget.vaisSpeechAudio.addListener(VaisSpeechCallback(
        onGeneratedSpeechText: (text) {
          print("Streaming text $text");
          setState(() {
            _text = text;
          });
        },
        onDurationChanged: (duration) {
          print("Duration changed: $duration");
          setState(() {
            _duration = duration;
          });
        },
        onConnectionChanged: (isOnConnection) {
          if (!isOnConnection) {
            print("Connection closed");
          }
        }
    ),singleUtterance: false);

    recordingState = RecordingState.initilized;
  }

  @override
  void dispose() {
    widget.vaisSpeechAudio.dispose();
    super.dispose();
  }


  @override
  Widget build(BuildContext context) {
    final mContext = context;
    return Scaffold(
      appBar: AppBar(
        backgroundColor: Colors.blue,
        title: Text('Thu âm'),
      ),
      body: SingleChildScrollView(
        child: Container(
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Container(
                  height: MediaQuery.of(context).size.height / 2.3,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: <Widget>[
                      widget.isOfflineMode ?
                      Padding(
                        padding: const EdgeInsets.all(12.0),
                        child: Text("Timing : ${_duration?.inSeconds}", style: TextStyle(color: Colors.white, fontSize: 48),),
                      ):
                      Column(
                        children: <Widget>[
                          Padding(
                            padding: const EdgeInsets.all(12.0),
                            child: Text("Timing : ${_duration?.inSeconds}", style: TextStyle(color: Colors.white, fontSize: 48),),
                          ),
                          DisplayText(text: _text,)
                        ],
                      ),
                    ],
                  ),
                  color: Colors.blue),
              !widget.isOfflineMode && recordingState == RecordingState.stopped ? Container() : Column(
                children: <Widget>[
                  Container(
                    margin: EdgeInsets.only(top: 32.0),
                    child: RaisedButton(
                      child: Text(recordingState != RecordingState.playing ? "Recording" : "Pause"),
                      onPressed: () async {
                        print(recordingState);
                        switch (recordingState) {
                          case RecordingState.playing:
                            _pauseRecording();
                            break;
                          case RecordingState.paused:
                            _resumeRecording();
                            break;
                          default:
                            bool hasPermission = await VaisSpeechAudio.hasPermissions;
                            if (hasPermission) {
                              _startRecording();
                            } else {
                              Toast.show('Lack of Permission in Manifest.', context);
                            }
                        }
                      },
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.all(12.0),
                    child: Text(_getRecordText(recordingState),
                        style: TextStyle(
                            color: Colors.redAccent,
                            fontWeight: FontWeight.w600)),
                  ),
                  recordingState != RecordingState.stopped && recordingState != RecordingState.initilized ? FlatButton(
                    child: Container(
                        margin: EdgeInsets.only(
                            left: 32.0, right: 32.0, top: 28.0),
                        decoration: BoxDecoration(
                            color: Colors.redAccent,
                            borderRadius:
                            BorderRadius.all(Radius.circular(12.0))),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: <Widget>[
                            Padding(
                              padding: const EdgeInsets.all(16.0),
                              child: Text("Ngừng ghi âm",
                                  style: TextStyle(
                                      color: Colors.white,
                                      fontWeight: FontWeight.w600)),
                            ),
                          ],
                        )),
                    onPressed: () {
                      _stopRecording();
                    },
                  ) : Container()
                ],
              )
            ],
          ),
        ),
      )
    );
  }

  String _getRecordText(RecordingState recordingState) {
    switch (recordingState) {
      case RecordingState.playing:
        return "Tạm ngưng";
      case RecordingState.paused:
        return "Tiếp tục";
      default:
        return "Bắt đầu";
    }
  }

  void _startRecording() async {
    await widget.vaisSpeechAudio.start();
    setState(() {
      recordingState = RecordingState.playing;
    });
  }

  void _pauseRecording() async {
    await widget.vaisSpeechAudio.pause();
    setState(() {
      recordingState = RecordingState.paused;
    });
  }

  void _resumeRecording() async {
    await widget.vaisSpeechAudio.resume();
    setState(() {
      recordingState = RecordingState.playing;
    });
  }

  void _stopRecording() async {
    var result = await widget.vaisSpeechAudio.stop();
    setState(() {
      recordingState = RecordingState.stopped;
    });
    print("Stop recording: ${result.path}");
    print("Stop recording: ${result.duration}");
    if (widget.isOfflineMode) {
      final audioId = await widget.vaisSpeech.startAsr(_path);
      Toast.show("Upload success with Audio ID: $audioId", context, duration: 3);
    } else {
      Navigator.of(context);
    }
  }

  Future<String> getPath() async {
    Directory systemDirectory;
    if (Platform.isIOS) {
      systemDirectory = await getApplicationDocumentsDirectory();
    } else if (Platform.isAndroid){
      systemDirectory = await getExternalStorageDirectory();
    }
    await Directory(systemDirectory.path).create(recursive: true);
    return systemDirectory.path +  "/${DateTime.now().toIso8601String()}_$name";
  }
}

class DisplayText extends StatelessWidget {
  final String text;

  const DisplayText({Key key, this.text = ""}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Text((text?.isNotEmpty ?? false) ? text : "Welcome to streaming mode"),
    );
  }
}
